@@ -1,18 +1,14 @@
 //! This is the main file to house data collection functions.
 
-use std::time::Instant;
-
-use futures::join;
+use std::time::{Duration, Instant};
 
 #[cfg(target_os = "linux")]
-use fxhash::FxHashMap;
-
+use hashbrown::HashMap;
 #[cfg(feature = "battery")]
 use starship_battery::{Battery, Manager};
-
-#[cfg(not(target_os = "linux"))]
 use sysinfo::{System, SystemExt};
 
+use self::temperature::TemperatureType;
 use super::DataFilters;
 use crate::app::layout_manager::UsedWidgets;
 
@@ -21,6 +17,7 @@ pub mod nvidia;
 
 #[cfg(feature = "battery")]
 pub mod batteries;
+
 pub mod cpu;
 pub mod disks;
 pub mod memory;
@@ -34,6 +31,8 @@ pub struct Data {
     pub cpu: Option<cpu::CpuHarvest>,
     pub load_avg: Option<cpu::LoadAvgHarvest>,
     pub memory: Option<memory::MemHarvest>,
+    #[cfg(not(target_os = "windows"))]
+    pub cache: Option<memory::MemHarvest>,
     pub swap: Option<memory::MemHarvest>,
     pub temperature_sensors: Option<Vec<temperature::TempHarvest>>,
     pub network: Option<network::NetworkHarvest>,
@@ -55,6 +54,8 @@ impl Default for Data {
             cpu: None,
             load_avg: None,
             memory: None,
+            #[cfg(not(target_os = "windows"))]
+            cache: None,
             swap: None,
             temperature_sensors: None,
             list_of_processes: None,
@@ -99,18 +100,8 @@ impl Data {
 #[derive(Debug)]
 pub struct DataCollector {
     pub data: Data,
-    #[cfg(not(target_os = "linux"))]
     sys: System,
-    previous_cpu_times: Vec<(cpu::PastCpuWork, cpu::PastCpuTotal)>,
-    previous_average_cpu_time: Option<(cpu::PastCpuWork, cpu::PastCpuTotal)>,
-    #[cfg(target_os = "linux")]
-    pid_mapping: FxHashMap<crate::Pid, processes::PrevProcDetails>,
-    #[cfg(target_os = "linux")]
-    prev_idle: f64,
-    #[cfg(target_os = "linux")]
-    prev_non_idle: f64,
-    mem_total_kb: u64,
-    temperature_type: temperature::TemperatureType,
+    temperature_type: TemperatureType,
     use_current_cpu_total: bool,
     unnormalized_cpu: bool,
     last_collection_time: Instant,
@@ -118,11 +109,19 @@ pub struct DataCollector {
     total_tx: u64,
     show_average_cpu: bool,
     widgets_to_harvest: UsedWidgets,
+    filters: DataFilters,
+
+    #[cfg(target_os = "linux")]
+    pid_mapping: HashMap<crate::Pid, processes::PrevProcDetails>,
+    #[cfg(target_os = "linux")]
+    prev_idle: f64,
+    #[cfg(target_os = "linux")]
+    prev_non_idle: f64,
+
     #[cfg(feature = "battery")]
     battery_manager: Option<Manager>,
     #[cfg(feature = "battery")]
     battery_list: Option<Vec<Battery>>,
-    filters: DataFilters,
 
     #[cfg(target_family = "unix")]
     user_table: self::processes::UserTable,
@@ -132,18 +131,14 @@ impl DataCollector {
     pub fn new(filters: DataFilters) -> Self {
         DataCollector {
             data: Data::default(),
-            #[cfg(not(target_os = "linux"))]
             sys: System::new_with_specifics(sysinfo::RefreshKind::new()),
-            previous_cpu_times: vec![],
-            previous_average_cpu_time: None,
             #[cfg(target_os = "linux")]
-            pid_mapping: FxHashMap::default(),
+            pid_mapping: HashMap::default(),
             #[cfg(target_os = "linux")]
             prev_idle: 0_f64,
             #[cfg(target_os = "linux")]
             prev_non_idle: 0_f64,
-            mem_total_kb: 0,
-            temperature_type: temperature::TemperatureType::Celsius,
+            temperature_type: TemperatureType::Celsius,
             use_current_cpu_total: false,
             unnormalized_cpu: false,
             last_collection_time: Instant::now(),
@@ -162,36 +157,6 @@ impl DataCollector {
     }
 
     pub fn init(&mut self) {
-        #[cfg(target_os = "linux")]
-        {
-            futures::executor::block_on(self.initialize_memory_size());
-        }
-        #[cfg(not(target_os = "linux"))]
-        {
-            self.sys.refresh_memory();
-            self.mem_total_kb = self.sys.total_memory();
-
-            // TODO: Would be good to get this and network list running on a timer instead...?
-            // Refresh components list once...
-            if self.widgets_to_harvest.use_temp {
-                self.sys.refresh_components_list();
-            }
-
-            // Refresh network list once...
-            if cfg!(target_os = "windows") && self.widgets_to_harvest.use_net {
-                self.sys.refresh_networks_list();
-            }
-
-            if self.widgets_to_harvest.use_proc || self.widgets_to_harvest.use_cpu {
-                self.sys.refresh_cpu();
-            }
-
-            // Refresh disk list once...
-            if cfg!(target_os = "freebsd") && self.widgets_to_harvest.use_disk {
-                self.sys.refresh_disks_list();
-            }
-        }
-
         #[cfg(feature = "battery")]
         {
             if self.widgets_to_harvest.use_battery {
@@ -207,29 +172,40 @@ impl DataCollector {
             }
         }
 
-        futures::executor::block_on(self.update_data());
+        // Sysinfo-related list refreshing.
+        if self.widgets_to_harvest.use_net {
+            self.sys.refresh_networks_list();
+        }
 
-        std::thread::sleep(std::time::Duration::from_millis(250));
+        if self.widgets_to_harvest.use_temp {
+            self.sys.refresh_components_list();
+        }
 
+        #[cfg(target_os = "windows")]
+        {
+            if self.widgets_to_harvest.use_proc {
+                self.sys.refresh_users_list();
+            }
+
+            if self.widgets_to_harvest.use_disk {
+                self.sys.refresh_disks_list();
+            }
+        }
+
+        self.update_data();
+
+        // Sleep a few seconds to avoid potentially weird data.
+        const SLEEP: Duration = get_sleep_duration();
+
+        std::thread::sleep(SLEEP);
         self.data.cleanup();
-
-        // trace!("Enabled widgets to harvest: {:#?}", self.widgets_to_harvest);
-    }
-
-    #[cfg(target_os = "linux")]
-    async fn initialize_memory_size(&mut self) {
-        self.mem_total_kb = if let Ok(mem) = heim::memory::memory().await {
-            mem.total().get::<heim::units::information::kilobyte>()
-        } else {
-            1
-        };
     }
 
     pub fn set_data_collection(&mut self, used_widgets: UsedWidgets) {
         self.widgets_to_harvest = used_widgets;
     }
 
-    pub fn set_temperature_type(&mut self, temperature_type: temperature::TemperatureType) {
+    pub fn set_temperature_type(&mut self, temperature_type: TemperatureType) {
         self.temperature_type = temperature_type;
     }
 
@@ -245,113 +221,136 @@ impl DataCollector {
         self.show_average_cpu = show_average_cpu;
     }
 
-    pub async fn update_data(&mut self) {
+    /// Refresh sysinfo data. We use sysinfo for the following data:
+    /// - CPU usage
+    /// - Memory usage
+    /// - Network usage
+    /// - Processes (non-Linux)
+    /// - Disk (Windows)
+    /// - Temperatures (non-Linux)
+    fn refresh_sysinfo_data(&mut self) {
+        // Refresh once every minute. If it's too frequent it can cause segfaults.
+        const LIST_REFRESH_TIME: Duration = Duration::from_secs(60);
+        let refresh_start = Instant::now();
+
+        if self.widgets_to_harvest.use_cpu || self.widgets_to_harvest.use_proc {
+            self.sys.refresh_cpu();
+        }
+
+        if self.widgets_to_harvest.use_mem || self.widgets_to_harvest.use_proc {
+            self.sys.refresh_memory();
+        }
+
+        if self.widgets_to_harvest.use_net {
+            if refresh_start.duration_since(self.last_collection_time) > LIST_REFRESH_TIME {
+                self.sys.refresh_networks_list();
+            }
+            self.sys.refresh_networks();
+        }
+
+        // sysinfo is used on non-Linux systems for the following:
+        // - Processes (users list as well for Windows)
+        // - Disks (Windows only)
+        // - Temperatures and temperature components list.
         #[cfg(not(target_os = "linux"))]
         {
-            if self.widgets_to_harvest.use_proc || self.widgets_to_harvest.use_cpu {
-                self.sys.refresh_cpu();
-            }
             if self.widgets_to_harvest.use_proc {
+                // For Windows, sysinfo also handles the users list.
+                #[cfg(target_os = "windows")]
+                if refresh_start.duration_since(self.last_collection_time) > LIST_REFRESH_TIME {
+                    self.sys.refresh_users_list();
+                }
+
                 self.sys.refresh_processes();
             }
+
             if self.widgets_to_harvest.use_temp {
+                if refresh_start.duration_since(self.last_collection_time) > LIST_REFRESH_TIME {
+                    self.sys.refresh_components_list();
+                }
                 self.sys.refresh_components();
             }
-
-            #[cfg(target_os = "windows")]
-            {
-                if self.widgets_to_harvest.use_net {
-                    self.sys.refresh_networks();
-                }
-            }
-
-            #[cfg(target_os = "freebsd")]
-            {
-                if self.widgets_to_harvest.use_disk {
-                    self.sys.refresh_disks();
-                }
-                if self.widgets_to_harvest.use_mem {
-                    self.sys.refresh_memory();
-                }
-            }
         }
 
-        let current_instant = std::time::Instant::now();
+        #[cfg(target_os = "windows")]
+        if self.widgets_to_harvest.use_disk {
+            if refresh_start.duration_since(self.last_collection_time) > LIST_REFRESH_TIME {
+                self.sys.refresh_disks_list();
+            }
+            self.sys.refresh_disks();
+        }
+    }
 
-        // CPU
+    pub fn update_data(&mut self) {
+        self.refresh_sysinfo_data();
+
+        let current_instant = Instant::now();
+
+        self.update_cpu_usage();
+        self.update_memory_usage();
+        self.update_processes(
+            #[cfg(target_os = "linux")]
+            current_instant,
+        );
+        self.update_temps();
+        self.update_network_usage(current_instant);
+        self.update_disks();
+
+        #[cfg(feature = "battery")]
+        self.update_batteries();
+
+        // Update times for future reference.
+        self.last_collection_time = current_instant;
+        self.data.last_collection_time = current_instant;
+    }
+
+    #[inline]
+    fn update_cpu_usage(&mut self) {
         if self.widgets_to_harvest.use_cpu {
-            #[cfg(not(target_os = "freebsd"))]
-            {
-                if let Ok(cpu_data) = cpu::get_cpu_data_list(
-                    self.show_average_cpu,
-                    &mut self.previous_cpu_times,
-                    &mut self.previous_average_cpu_time,
-                )
-                .await
-                {
-                    self.data.cpu = Some(cpu_data);
-                }
-            }
-            #[cfg(target_os = "freebsd")]
-            {
-                if let Ok(cpu_data) = cpu::get_cpu_data_list(
-                    &self.sys,
-                    self.show_average_cpu,
-                    &mut self.previous_cpu_times,
-                    &mut self.previous_average_cpu_time,
-                )
-                .await
-                {
-                    self.data.cpu = Some(cpu_data);
-                }
-            }
+            self.data.cpu = cpu::get_cpu_data_list(&self.sys, self.show_average_cpu).ok();
 
             #[cfg(target_family = "unix")]
             {
-                // Load Average
-                if let Ok(load_avg_data) = cpu::get_load_avg().await {
-                    self.data.load_avg = Some(load_avg_data);
-                }
-            }
-        }
-
-        // Batteries
-        #[cfg(feature = "battery")]
-        {
-            if let Some(battery_manager) = &self.battery_manager {
-                if let Some(battery_list) = &mut self.battery_list {
-                    self.data.list_of_batteries =
-                        Some(batteries::refresh_batteries(battery_manager, battery_list));
-                }
+                self.data.load_avg = cpu::get_load_avg().ok();
             }
         }
+    }
 
+    #[inline]
+    fn update_processes(&mut self, #[cfg(target_os = "linux")] current_instant: Instant) {
         if self.widgets_to_harvest.use_proc {
             if let Ok(mut process_list) = {
+                let total_memory = if let Some(memory) = &self.data.memory {
+                    memory.total_bytes
+                } else {
+                    self.sys.total_memory()
+                };
+
                 #[cfg(target_os = "linux")]
                 {
-                    // Must do this here since we otherwise have to make `get_process_data` async.
-                    use self::processes::CpuUsageStrategy;
+                    use self::processes::{PrevProc, ProcHarvestOptions};
 
-                    let normalize_cpu = if self.unnormalized_cpu {
-                        heim::cpu::logical_count()
-                            .await
-                            .map(|v| CpuUsageStrategy::NonNormalized(v as f64))
-                            .unwrap_or(CpuUsageStrategy::Normalized)
-                    } else {
-                        CpuUsageStrategy::Normalized
+                    let prev_proc = PrevProc {
+                        prev_idle: &mut self.prev_idle,
+                        prev_non_idle: &mut self.prev_non_idle,
                     };
 
+                    let proc_harvest_options = ProcHarvestOptions {
+                        use_current_cpu_total: self.use_current_cpu_total,
+                        unnormalized_cpu: self.unnormalized_cpu,
+                    };
+
+                    let time_diff = current_instant
+                        .duration_since(self.last_collection_time)
+                        .as_secs();
+
                     processes::get_process_data(
-                        &mut self.prev_idle,
-                        &mut self.prev_non_idle,
+                        &self.sys,
+                        prev_proc,
                         &mut self.pid_mapping,
-                        self.use_current_cpu_total,
-                        normalize_cpu,
-                        current_instant
-                            .duration_since(self.last_collection_time)
-                            .as_secs(),
-                        self.mem_total_kb,
+                        proc_harvest_options,
+                        time_diff,
+                        total_memory,
                         &mut self.user_table,
                     )
                 }
@@ -363,7 +362,7 @@ impl DataCollector {
                             &self.sys,
                             self.use_current_cpu_total,
                             self.unnormalized_cpu,
-                            self.mem_total_kb,
+                            total_memory,
                             &mut self.user_table,
                         )
                     }
@@ -373,7 +372,7 @@ impl DataCollector {
                             &self.sys,
                             self.use_current_cpu_total,
                             self.unnormalized_cpu,
-                            self.mem_total_kb,
+                            total_memory,
                         )
                     }
                 }
@@ -385,123 +384,128 @@ impl DataCollector {
                 self.data.list_of_processes = Some(process_list);
             }
         }
+    }
 
+    #[inline]
+    fn update_temps(&mut self) {
         if self.widgets_to_harvest.use_temp {
             #[cfg(not(target_os = "linux"))]
-            {
-                if let Ok(data) = temperature::get_temperature_data(
-                    &self.sys,
-                    &self.temperature_type,
-                    &self.filters.temp_filter,
-                ) {
-                    self.data.temperature_sensors = data;
-                }
+            if let Ok(data) = temperature::get_temperature_data(
+                &self.sys,
+                &self.temperature_type,
+                &self.filters.temp_filter,
+            ) {
+                self.data.temperature_sensors = data;
             }
 
             #[cfg(target_os = "linux")]
+            if let Ok(data) =
+                temperature::get_temperature_data(&self.temperature_type, &self.filters.temp_filter)
             {
-                if let Ok(data) = temperature::get_temperature_data(
-                    &self.temperature_type,
-                    &self.filters.temp_filter,
-                ) {
-                    self.data.temperature_sensors = data;
-                }
+                self.data.temperature_sensors = data;
             }
         }
+    }
 
-        let network_data_fut = {
-            #[cfg(any(target_os = "windows", target_os = "freebsd"))]
+    #[inline]
+    fn update_memory_usage(&mut self) {
+        if self.widgets_to_harvest.use_mem {
+            self.data.memory = memory::get_ram_usage(&self.sys);
+
+            #[cfg(not(target_os = "windows"))]
+            if self.widgets_to_harvest.use_cache {
+                self.data.cache = memory::get_cache_usage(&self.sys);
+            }
+
+            self.data.swap = memory::get_swap_usage(
+                #[cfg(not(target_os = "windows"))]
+                &self.sys,
+            );
+
+            #[cfg(feature = "zfs")]
             {
-                network::get_network_data(
+                self.data.arc = memory::arc::get_arc_usage();
+            }
+
+            #[cfg(feature = "gpu")]
+            if self.widgets_to_harvest.use_gpu {
+                self.data.gpu = memory::gpu::get_gpu_mem_usage();
+            }
+        }
+    }
+
+    #[inline]
+    fn update_network_usage(&mut self, current_instant: Instant) {
+        if self.widgets_to_harvest.use_net {
+            let net_data = network::get_network_data(
+                &self.sys,
+                self.last_collection_time,
+                &mut self.total_rx,
+                &mut self.total_tx,
+                current_instant,
+                &self.filters.net_filter,
+            );
+
+            self.total_rx = net_data.total_rx;
+            self.total_tx = net_data.total_tx;
+            self.data.network = Some(net_data);
+        }
+    }
+
+    #[inline]
+    #[cfg(feature = "battery")]
+    fn update_batteries(&mut self) {
+        if let Some(battery_manager) = &self.battery_manager {
+            if let Some(battery_list) = &mut self.battery_list {
+                self.data.list_of_batteries =
+                    Some(batteries::refresh_batteries(battery_manager, battery_list));
+            }
+        }
+    }
+
+    #[inline]
+    fn update_disks(&mut self) {
+        if self.widgets_to_harvest.use_disk {
+            #[cfg(any(target_os = "freebsd", target_os = "linux", target_os = "macos"))]
+            {
+                let disk_filter = &self.filters.disk_filter;
+                let mount_filter = &self.filters.mount_filter;
+                self.data.disks = disks::get_disk_usage(disk_filter, mount_filter).ok();
+            }
+
+            #[cfg(target_os = "windows")]
+            {
+                self.data.disks = Some(disks::get_disk_usage(
                     &self.sys,
-                    self.last_collection_time,
-                    &mut self.total_rx,
-                    &mut self.total_tx,
-                    current_instant,
-                    self.widgets_to_harvest.use_net,
-                    &self.filters.net_filter,
-                )
+                    &self.filters.disk_filter,
+                    &self.filters.mount_filter,
+                ));
             }
-            #[cfg(not(any(target_os = "windows", target_os = "freebsd")))]
-            {
-                network::get_network_data(
-                    self.last_collection_time,
-                    &mut self.total_rx,
-                    &mut self.total_tx,
-                    current_instant,
-                    self.widgets_to_harvest.use_net,
-                    &self.filters.net_filter,
-                )
-            }
-        };
-        let mem_data_fut = {
-            #[cfg(not(target_os = "freebsd"))]
-            {
-                memory::get_mem_data(
-                    self.widgets_to_harvest.use_mem,
-                    self.widgets_to_harvest.use_gpu,
-                )
-            }
-            #[cfg(target_os = "freebsd")]
-            {
-                memory::get_mem_data(
-                    &self.sys,
-                    self.widgets_to_harvest.use_mem,
-                    self.widgets_to_harvest.use_gpu,
-                )
-            }
-        };
-        let disk_data_fut = disks::get_disk_usage(
-            self.widgets_to_harvest.use_disk,
-            &self.filters.disk_filter,
-            &self.filters.mount_filter,
-        );
-        let disk_io_usage_fut = disks::get_io_usage(self.widgets_to_harvest.use_disk);
 
-        let (net_data, mem_res, disk_res, io_res) = join!(
-            network_data_fut,
-            mem_data_fut,
-            disk_data_fut,
-            disk_io_usage_fut,
-        );
-
-        if let Ok(net_data) = net_data {
-            if let Some(net_data) = &net_data {
-                self.total_rx = net_data.total_rx;
-                self.total_tx = net_data.total_tx;
-            }
-            self.data.network = net_data;
+            self.data.io = disks::get_io_usage().ok();
         }
+    }
+}
 
-        if let Ok(memory) = mem_res.ram {
-            self.data.memory = memory;
-        }
+/// We set a sleep duration between 10ms and 250ms, ideally sysinfo's [`System::MINIMUM_CPU_UPDATE_INTERVAL`] + 1.
+///
+/// We bound the upper end to avoid waiting too long (e.g. FreeBSD is 1s, which I'm fine with losing
+/// accuracy on for the first refresh), and we bound the lower end just to avoid the off-chance that
+/// refreshing too quickly causes problems. This second case should only happen on unsupported
+/// systems via sysinfo, in which case [`System::MINIMUM_CPU_UPDATE_INTERVAL`] is defined as 0.
+///
+/// We also do `INTERVAL + 1` for some wiggle room, just in case.
+const fn get_sleep_duration() -> Duration {
+    const MIN_SLEEP: u64 = 10;
+    const MAX_SLEEP: u64 = 250;
+    const INTERVAL: u64 = System::MINIMUM_CPU_UPDATE_INTERVAL.as_millis() as u64;
 
-        if let Ok(swap) = mem_res.swap {
-            self.data.swap = swap;
-        }
-
-        #[cfg(feature = "zfs")]
-        if let Ok(arc) = mem_res.arc {
-            self.data.arc = arc;
-        }
-
-        #[cfg(feature = "gpu")]
-        if let Ok(gpu) = mem_res.gpus {
-            self.data.gpu = gpu;
-        }
-
-        if let Ok(disks) = disk_res {
-            self.data.disks = disks;
-        }
-
-        if let Ok(io) = io_res {
-            self.data.io = io;
-        }
-
-        // Update time
-        self.data.last_collection_time = current_instant;
-        self.last_collection_time = current_instant;
+    if INTERVAL < MIN_SLEEP {
+        Duration::from_millis(MIN_SLEEP)
+    } else if INTERVAL > MAX_SLEEP {
+        Duration::from_millis(MAX_SLEEP)
+    } else {
+        Duration::from_millis(INTERVAL + 1)
     }
 }
 
