@@ -1,5 +1,6 @@
 //! Process data collection for macOS.  Uses sysinfo and custom bindings.
 
+use hashbrown::HashMap;
 use sysinfo::System;
 
 use super::ProcessHarvest;
@@ -7,14 +8,14 @@ use crate::{data_harvester::processes::UserTable, Pid};
 mod sysctl_bindings;
 
 pub fn get_process_data(
-    sys: &System, use_current_cpu_total: bool, unnormalized_cpu: bool, mem_total_kb: u64,
+    sys: &System, use_current_cpu_total: bool, unnormalized_cpu: bool, mem_total: u64,
     user_table: &mut UserTable,
 ) -> crate::utils::error::Result<Vec<ProcessHarvest>> {
     super::macos_freebsd::get_process_data(
         sys,
         use_current_cpu_total,
         unnormalized_cpu,
-        mem_total_kb,
+        mem_total,
         user_table,
         get_macos_process_cpu_usage,
     )
@@ -26,9 +27,7 @@ pub(crate) fn fallback_macos_ppid(pid: Pid) -> Option<Pid> {
         .ok()
 }
 
-fn get_macos_process_cpu_usage(
-    pids: &[Pid],
-) -> std::io::Result<std::collections::HashMap<i32, f64>> {
+fn get_macos_process_cpu_usage(pids: &[Pid]) -> std::io::Result<HashMap<i32, f64>> {
     use itertools::Itertools;
     let output = std::process::Command::new("ps")
         .args(["-o", "pid=,pcpu=", "-p"])
@@ -38,7 +37,7 @@ fn get_macos_process_cpu_usage(
                 .collect::<String>(),
         )
         .output()?;
-    let mut result = std::collections::HashMap::new();
+    let mut result = HashMap::new();
     String::from_utf8_lossy(&output.stdout)
         .split_whitespace()
         .chunks(2)
