@@ -1,12 +1,12 @@
 //! Unix-specific parts of process collection.
 
-use fxhash::FxHashMap;
+use hashbrown::HashMap;
 
 use crate::utils::error;
 
 #[derive(Debug, Default)]
 pub struct UserTable {
-    pub uid_user_mapping: FxHashMap<libc::uid_t, String>,
+    pub uid_user_mapping: HashMap<libc::uid_t, String>,
 }
 
 impl UserTable {
@@ -18,15 +18,16 @@ impl UserTable {
             let passwd = unsafe { libc::getpwuid(uid) };
 
             if passwd.is_null() {
-                return Err(error::BottomError::QueryError("Missing passwd".into()));
+                Err(error::BottomError::QueryError("Missing passwd".into()))
+            } else {
+                // SAFETY: We return early if passwd is null.
+                let username = unsafe { std::ffi::CStr::from_ptr((*passwd).pw_name) }
+                    .to_str()?
+                    .to_string();
+                self.uid_user_mapping.insert(uid, username.clone());
+
+                Ok(username)
             }
-
-            let username = unsafe { std::ffi::CStr::from_ptr((*passwd).pw_name) }
-                .to_str()?
-                .to_string();
-            self.uid_user_mapping.insert(uid, username.clone());
-
-            Ok(username)
         }
     }
 }
