@@ -1,5 +1,10 @@
 #![warn(rust_2018_idioms)]
-#[allow(unused_imports)]
+#![deny(clippy::todo)]
+#![deny(clippy::unimplemented)]
+#![deny(clippy::missing_safety_doc)]
+#[allow(unused_imports)] // TODO: Deny unused imports.
+
+// Primarily used for debug purposes.
 #[cfg(feature = "log")]
 #[macro_use]
 extern crate log;
@@ -17,13 +22,6 @@ use std::{
 };
 
 use anyhow::{Context, Result};
-use bottom::{
-    canvas::{self, canvas_colours::CanvasColours},
-    constants::*,
-    data_conversion::*,
-    options::*,
-    *,
-};
 use crossterm::{
     event::{EnableBracketedPaste, EnableMouseCapture},
     execute,
@@ -31,18 +29,33 @@ use crossterm::{
 };
 use tui::{backend::CrosstermBackend, Terminal};
 
+use bottom::{
+    canvas::{self, canvas_styling::CanvasColours},
+    constants::*,
+    data_conversion::*,
+    options::*,
+    *,
+};
+
+// Used for heap allocation debugging purposes.
+// #[global_allocator]
+// static ALLOC: dhat::Alloc = dhat::Alloc;
+
 fn main() -> Result<()> {
+    // let _profiler = dhat::Profiler::new_heap();
+
     let matches = clap::get_matches();
-    #[cfg(all(feature = "fern", debug_assertions))]
+    #[cfg(all(feature = "fern"))]
     {
-        utils::logging::init_logger(log::LevelFilter::Debug, std::ffi::OsStr::new("debug.log"))?;
+        if let Err(err) =
+            utils::logging::init_logger(log::LevelFilter::Debug, std::ffi::OsStr::new("debug.log"))
+        {
+            println!("Issue initializing logger: {err}");
+        }
     }
 
-    // Check if the current environment is in a terminal.
-    check_if_terminal();
-
     // Read from config file.
-    let config_path = read_config(matches.value_of("config_location"))
+    let config_path = read_config(matches.get_one::<String>("config_location"))
         .context("Unable to access the given config file location.")?;
     let mut config: Config = create_or_get_config(&config_path)
         .context("Unable to properly parse or create the config file.")?;
@@ -58,7 +71,7 @@ fn main() -> Result<()> {
         CanvasColours::new(colour_scheme, &config)?
     };
 
-    // Create "app" struct, which will control most of the program and store settings/state
+    // Create an "app" struct, which will control most of the program and store settings/state
     let mut app = build_app(
         &matches,
         &mut config,
@@ -71,20 +84,36 @@ fn main() -> Result<()> {
     // Create painter and set colours.
     let mut painter = canvas::Painter::init(widget_layout, colours)?;
 
+    // Check if the current environment is in a terminal.
+    check_if_terminal();
+
     // Create termination mutex and cvar
     #[allow(clippy::mutex_atomic)]
     let thread_termination_lock = Arc::new(Mutex::new(false));
     let thread_termination_cvar = Arc::new(Condvar::new());
 
-    // Set up input handling
     let (sender, receiver) = mpsc::channel();
+
+    // Set up event loop thread; we set this up early to speed up first-time-to-data.
+    let (collection_thread_ctrl_sender, collection_thread_ctrl_receiver) = mpsc::channel();
+    let _collection_thread = create_collection_thread(
+        sender.clone(),
+        collection_thread_ctrl_receiver,
+        thread_termination_lock.clone(),
+        thread_termination_cvar.clone(),
+        &app.app_config_fields,
+        app.filters.clone(),
+        app.used_widgets,
+    );
+
+    // Set up input handling loop thread.
     let _input_thread = create_input_thread(sender.clone(), thread_termination_lock.clone());
 
-    // Cleaning loop
+    // Set up cleaning loop thread.
     let _cleaning_thread = {
         let lock = thread_termination_lock.clone();
         let cvar = thread_termination_cvar.clone();
-        let cleaning_sender = sender.clone();
+        let cleaning_sender = sender;
         let offset_wait_time = app.app_config_fields.retention_ms + 60000;
         thread::spawn(move || {
             loop {
@@ -105,19 +134,7 @@ fn main() -> Result<()> {
         })
     };
 
-    // Event loop
-    let (collection_thread_ctrl_sender, collection_thread_ctrl_receiver) = mpsc::channel();
-    let _collection_thread = create_collection_thread(
-        sender,
-        collection_thread_ctrl_receiver,
-        thread_termination_lock.clone(),
-        thread_termination_cvar.clone(),
-        &app.app_config_fields,
-        app.filters.clone(),
-        app.used_widgets.clone(),
-    );
-
-    // Set up up tui and crossterm
+    // Set up tui and crossterm
     let mut stdout_val = stdout();
     execute!(
         stdout_val,
@@ -134,9 +151,9 @@ fn main() -> Result<()> {
     #[cfg(target_os = "freebsd")]
     let _stderr_fd = {
         // A really ugly band-aid to suppress stderr warnings on FreeBSD due to sysinfo.
-        use std::fs::OpenOptions;
-
+        // For more information, see https://github.com/ClementTsang/bottom/issues/798.
         use filedescriptor::{FileDescriptor, StdioDescriptor};
+        use std::fs::OpenOptions;
 
         let path = OpenOptions::new().write(true).open("/dev/null")?;
         FileDescriptor::redirect_stdio(&path, StdioDescriptor::Stderr)?
@@ -153,12 +170,16 @@ fn main() -> Result<()> {
     })?;
     let mut first_run = true;
 
+    // Draw once first to initialize the canvas, so it doesn't feel like it's frozen.
+    try_drawing(&mut terminal, &mut app, &mut painter)?;
+
     while !is_terminated.load(Ordering::SeqCst) {
         // TODO: Would be good to instead use a mix of is_terminated check + recv. Probably use a termination event instead.
         if let Ok(recv) = receiver.recv_timeout(Duration::from_millis(TICK_RATE_IN_MILLISECONDS)) {
             match recv {
                 BottomEvent::Resize => {
-                    try_drawing(&mut terminal, &mut app, &mut painter)?; // FIXME: This is bugged with frozen?
+                    // FIXME: This is bugged with frozen?
+                    try_drawing(&mut terminal, &mut app, &mut painter)?;
                 }
                 BottomEvent::KeyInput(event) => {
                     if handle_key_event_or_break(event, &mut app, &collection_thread_ctrl_sender) {
@@ -237,6 +258,11 @@ fn main() -> Result<()> {
                         if app.used_widgets.use_mem {
                             app.converted_data.mem_data =
                                 convert_mem_data_points(&app.data_collection);
+                            #[cfg(not(target_os = "windows"))]
+                            {
+                                app.converted_data.cache_data =
+                                    convert_cache_data_points(&app.data_collection);
+                            }
                             app.converted_data.swap_data =
                                 convert_swap_data_points(&app.data_collection);
                             #[cfg(feature = "zfs")]
@@ -249,11 +275,17 @@ fn main() -> Result<()> {
                                 app.converted_data.gpu_data =
                                     convert_gpu_data(&app.data_collection);
                             }
-                            let (memory_labels, swap_labels) =
-                                convert_mem_labels(&app.data_collection);
 
-                            app.converted_data.mem_labels = memory_labels;
-                            app.converted_data.swap_labels = swap_labels;
+                            app.converted_data.mem_labels =
+                                convert_mem_label(&app.data_collection.memory_harvest);
+                            app.converted_data.swap_labels =
+                                convert_mem_label(&app.data_collection.swap_harvest);
+                            #[cfg(not(target_os = "windows"))]
+                            {
+                                app.converted_data.cache_labels =
+                                    convert_mem_label(&app.data_collection.cache_harvest);
+                            }
+
                             #[cfg(feature = "zfs")]
                             {
                                 let arc_labels = convert_arc_labels(&app.data_collection);
@@ -296,11 +328,8 @@ fn main() -> Result<()> {
     }
 
     // I think doing it in this order is safe...
-
     *thread_termination_lock.lock().unwrap() = true;
-
     thread_termination_cvar.notify_all();
-
     cleanup_terminal(&mut terminal)?;
 
     Ok(())
