@@ -1,6 +1,6 @@
 use std::str::FromStr;
 
-use canvas_colours::*;
+use canvas_styling::*;
 use itertools::izip;
 use tui::{
     backend::Backend,
@@ -21,7 +21,7 @@ use crate::{
     utils::error::BottomError,
 };
 
-pub mod canvas_colours;
+pub mod canvas_styling;
 mod dialogs;
 mod drawing_utils;
 mod widgets;
@@ -435,6 +435,13 @@ impl Painter {
                     }
                 }
 
+                #[cfg(not(target_os = "windows"))]
+                {
+                    if app_state.converted_data.cache_labels.is_some() {
+                        mem_rows += 1;
+                    }
+                }
+
                 #[cfg(feature = "gpu")]
                 {
                     if let Some(gpu_data) = &app_state.converted_data.gpu_data {
