@@ -1,4 +1,5 @@
 use once_cell::sync::Lazy;
+use tui::widgets::Borders;
 
 use crate::options::ConfigColours;
 
@@ -21,7 +22,7 @@ pub const TABLE_GAP_HEIGHT_LIMIT: u16 = 7;
 pub const TIME_LABEL_HEIGHT_LIMIT: u16 = 7;
 
 // Side borders
-pub const SIDE_BORDERS: tui::widgets::Borders = tui::widgets::Borders::from_bits_truncate(20);
+pub const SIDE_BORDERS: Borders = Borders::LEFT.union(Borders::RIGHT);
 pub static DEFAULT_TEXT_STYLE: Lazy<tui::style::Style> =
     Lazy::new(|| tui::style::Style::default().fg(tui::style::Color::Gray));
 pub static DEFAULT_HEADER_STYLE: Lazy<tui::style::Style> =
@@ -37,6 +38,8 @@ pub static DEFAULT_LIGHT_MODE_COLOUR_PALETTE: Lazy<ConfigColours> = Lazy::new(||
     graph_color: Some("black".into()),
     disabled_text_color: Some("gray".into()),
     ram_color: Some("blue".into()),
+    #[cfg(not(target_os = "windows"))]
+    cache_color: Some("LightRed".into()),
     swap_color: Some("red".into()),
     arc_color: Some("LightBlue".into()),
     gpu_core_colors: Some(vec![
@@ -91,6 +94,8 @@ pub static GRUVBOX_COLOUR_PALETTE: Lazy<ConfigColours> = Lazy::new(|| ConfigColo
         "#af3a03".into(),
     ]),
     ram_color: Some("#8ec07c".into()),
+    #[cfg(not(target_os = "windows"))]
+    cache_color: Some("#b16286".into()),
     swap_color: Some("#fabd2f".into()),
     arc_color: Some("#689d6a".into()),
     gpu_core_colors: Some(vec![
@@ -146,6 +151,8 @@ pub static GRUVBOX_LIGHT_COLOUR_PALETTE: Lazy<ConfigColours> = Lazy::new(|| Conf
         "#af3a03".into(),
     ]),
     ram_color: Some("#427b58".into()),
+    #[cfg(not(target_os = "windows"))]
+    cache_color: Some("d79921".into()),
     swap_color: Some("#cc241d".into()),
     arc_color: Some("#689d6a".into()),
     gpu_core_colors: Some(vec![
@@ -189,6 +196,8 @@ pub static NORD_COLOUR_PALETTE: Lazy<ConfigColours> = Lazy::new(|| ConfigColours
         "#bf616a".into(),
     ]),
     ram_color: Some("#88c0d0".into()),
+    #[cfg(not(target_os = "windows"))]
+    cache_color: Some("#d8dee9".into()),
     swap_color: Some("#d08770".into()),
     arc_color: Some("#5e81ac".into()),
     gpu_core_colors: Some(vec![
@@ -232,6 +241,8 @@ pub static NORD_LIGHT_COLOUR_PALETTE: Lazy<ConfigColours> = Lazy::new(|| ConfigC
         "#bf616a".into(),
     ]),
     ram_color: Some("#81a1c1".into()),
+    #[cfg(not(target_os = "windows"))]
+    cache_color: Some("#4c566a".into()),
     swap_color: Some("#d08770".into()),
     arc_color: Some("#5e81ac".into()),
     gpu_core_colors: Some(vec![
@@ -356,11 +367,11 @@ pub const SEARCH_HELP_TEXT: [&str; 48] = [
     "cpu, cpu%        ex: cpu > 4.2",
     "mem, mem%        ex: mem < 4.2",
     "memb             ex: memb < 100 kb",
-    "read, r/s        ex: read >= 1 b",
-    "write, w/s       ex: write <= 1 tb",
+    "read, r/s, rps   ex: read >= 1 b",
+    "write, w/s, wps  ex: write <= 1 tb",
     "tread, t.read    ex: tread = 1",
     "twrite, t.write  ex: twrite = 1",
-    "user            ex: user = root",
+    "user             ex: user = root",
     "state            ex: state = running",
     "",
     "Comparison operators:",
@@ -494,7 +505,7 @@ pub const DEFAULT_BATTERY_LAYOUT: &str = r##"
 // Config and flags
 pub const DEFAULT_CONFIG_FILE_PATH: &str = "bottom/bottom.toml";
 
-// TODO: Eventually deprecate this.
+// TODO: Eventually deprecate this, or grab from a file.
 pub const CONFIG_TEXT: &str = r##"# This is a default config file for bottom.  All of the settings are commented
 # out by default; if you wish to change them uncomment and modify as you see
 # fit.
@@ -572,9 +583,16 @@ pub const CONFIG_TEXT: &str = r##"# This is a default config file for bottom.  A
 #disable_advanced_kill = false
 # Shows GPU(s) memory
 #enable_gpu_memory = false
+# Shows cache and buffer memory
+#enable_cache_memory = false
 # How much data is stored at once in terms of time.
 #retention = "10m"
 
+# These are flags around the process widget.
+
+#[processes]
+#columns = ["PID", "Name", "CPU%", "Mem%", "R/s", "W/s", "T.Read", "T.Write", "User", "State"]
+
 # These are all the components that support custom theming.  Note that colour support
 # will depend on terminal support.
 
@@ -726,4 +744,14 @@ mod test {
             "the two should be equal, or this test should be updated"
         )
     }
+
+    /// This test exists because previously, [`SIDE_BORDERS`] was set incorrectly after I moved from
+    /// tui-rs to ratatui.
+    #[test]
+    fn assert_side_border_bits_match() {
+        assert_eq!(
+            SIDE_BORDERS,
+            Borders::ALL.difference(Borders::TOP.union(Borders::BOTTOM))
+        )
+    }
 }
