@@ -3,6 +3,7 @@
 
 use kstring::KString;
 
+use crate::app::data_harvester::memory::MemHarvest;
 use crate::app::{
     data_farmer::DataCollection,
     data_harvester::{cpu::CpuDataType, temperature::TemperatureType},
@@ -13,19 +14,16 @@ use crate::units::data_units::DataUnit;
 use crate::utils::gen_util::*;
 use crate::widgets::{DiskWidgetData, TempWidgetData};
 
-#[derive(Debug)]
+#[derive(Debug, Default)]
 pub enum BatteryDuration {
     ToEmpty(i64),
     ToFull(i64),
+    Empty,
+    Full,
+    #[default]
     Unknown,
 }
 
-impl Default for BatteryDuration {
-    fn default() -> Self {
-        BatteryDuration::Unknown
-    }
-}
-
 #[derive(Default, Debug)]
 pub struct ConvertedBatteryData {
     pub battery_name: String,
@@ -33,6 +31,7 @@ pub struct ConvertedBatteryData {
     pub watt_consumption: String,
     pub battery_duration: BatteryDuration,
     pub health: String,
+    pub state: String,
 }
 
 #[derive(Default, Debug)]
@@ -73,9 +72,13 @@ pub struct ConvertedData {
     pub network_data_tx: Vec<Point>,
 
     pub mem_labels: Option<(String, String)>,
+    #[cfg(not(target_os = "windows"))]
+    pub cache_labels: Option<(String, String)>,
     pub swap_labels: Option<(String, String)>,
 
     pub mem_data: Vec<Point>, /* TODO: Switch this and all data points over to a better data structure... */
+    #[cfg(not(target_os = "windows"))]
+    pub cache_data: Vec<Point>,
     pub swap_data: Vec<Point>,
 
     #[cfg(feature = "zfs")]
@@ -102,6 +105,7 @@ impl ConvertedData {
             .iter()
             .zip(&data.io_labels)
             .for_each(|(disk, (io_read, io_write))| {
+                // Because this sometimes does *not* equal to disk.total.
                 let summed_total_bytes = match (disk.used_space, disk.free_space) {
                     (Some(used), Some(free)) => Some(used + free),
                     _ => None,
@@ -223,6 +227,25 @@ pub fn convert_mem_data_points(current_data: &DataCollection) -> Vec<Point> {
     result
 }
 
+#[cfg(not(target_os = "windows"))]
+pub fn convert_cache_data_points(current_data: &DataCollection) -> Vec<Point> {
+    let mut result: Vec<Point> = Vec::new();
+    let current_time = current_data.current_instant;
+
+    for (time, data) in &current_data.timed_data_vec {
+        if let Some(cache_data) = data.cache_data {
+            let time_from_start: f64 =
+                (current_time.duration_since(*time).as_millis() as f64).floor();
+            result.push((-time_from_start, cache_data));
+            if *time == current_time {
+                break;
+            }
+        }
+    }
+
+    result
+}
+
 pub fn convert_swap_data_points(current_data: &DataCollection) -> Vec<Point> {
     let mut result: Vec<Point> = Vec::new();
     let current_time = current_data.current_instant;
@@ -241,92 +264,60 @@ pub fn convert_swap_data_points(current_data: &DataCollection) -> Vec<Point> {
     result
 }
 
-pub fn convert_mem_labels(
-    current_data: &DataCollection,
-) -> (Option<(String, String)>, Option<(String, String)>) {
-    /// Returns the unit type and denominator for given total amount of memory in kibibytes.
-    fn return_unit_and_denominator_for_mem_kib(mem_total_kib: u64) -> (&'static str, f64) {
-        if mem_total_kib < 1024 {
-            // Stay with KiB
-            ("KiB", 1.0)
-        } else if mem_total_kib < MEBI_LIMIT {
-            // Use MiB
-            ("MiB", KIBI_LIMIT_F64)
-        } else if mem_total_kib < GIBI_LIMIT {
-            // Use GiB
-            ("GiB", MEBI_LIMIT_F64)
-        } else {
-            // Use TiB
-            ("TiB", GIBI_LIMIT_F64)
-        }
+/// Returns the most appropriate binary prefix unit type (e.g. kibibyte) and denominator for the given amount of bytes.
+///
+/// The expected usage is to divide out the given value with the returned denominator in order to be able to use it
+/// with the returned binary unit (e.g. divide 3000 bytes by 1024 to have a value in KiB).
+fn get_mem_binary_unit_and_denominator(bytes: u64) -> (&'static str, f64) {
+    if bytes < KIBI_LIMIT {
+        // Stick with bytes if under a kibibyte.
+        ("B", 1.0)
+    } else if bytes < MEBI_LIMIT {
+        ("KiB", KIBI_LIMIT_F64)
+    } else if bytes < GIBI_LIMIT {
+        ("MiB", MEBI_LIMIT_F64)
+    } else if bytes < TEBI_LIMIT {
+        ("GiB", GIBI_LIMIT_F64)
+    } else {
+        // Otherwise just use tebibytes, which is probably safe for most use cases.
+        ("TiB", TEBI_LIMIT_F64)
     }
+}
 
-    (
-        if current_data.memory_harvest.mem_total_in_kib > 0 {
-            Some((
-                format!(
-                    "{:3.0}%",
-                    current_data.memory_harvest.use_percent.unwrap_or(0.0)
-                ),
-                {
-                    let (unit, denominator) = return_unit_and_denominator_for_mem_kib(
-                        current_data.memory_harvest.mem_total_in_kib,
-                    );
+/// Returns the unit type and denominator for given total amount of memory in kibibytes.
+pub fn convert_mem_label(harvest: &MemHarvest) -> Option<(String, String)> {
+    if harvest.total_bytes > 0 {
+        Some((format!("{:3.0}%", harvest.use_percent.unwrap_or(0.0)), {
+            let (unit, denominator) = get_mem_binary_unit_and_denominator(harvest.total_bytes);
 
-                    format!(
-                        "   {:.1}{}/{:.1}{}",
-                        current_data.memory_harvest.mem_used_in_kib as f64 / denominator,
-                        unit,
-                        (current_data.memory_harvest.mem_total_in_kib as f64 / denominator),
-                        unit
-                    )
-                },
-            ))
-        } else {
-            None
-        },
-        if current_data.swap_harvest.mem_total_in_kib > 0 {
-            Some((
-                format!(
-                    "{:3.0}%",
-                    current_data.swap_harvest.use_percent.unwrap_or(0.0)
-                ),
-                {
-                    let (unit, denominator) = return_unit_and_denominator_for_mem_kib(
-                        current_data.swap_harvest.mem_total_in_kib,
-                    );
-
-                    format!(
-                        "   {:.1}{}/{:.1}{}",
-                        current_data.swap_harvest.mem_used_in_kib as f64 / denominator,
-                        unit,
-                        (current_data.swap_harvest.mem_total_in_kib as f64 / denominator),
-                        unit
-                    )
-                },
-            ))
-        } else {
-            None
-        },
-    )
+            format!(
+                "   {:.1}{}/{:.1}{}",
+                harvest.used_bytes as f64 / denominator,
+                unit,
+                (harvest.total_bytes as f64 / denominator),
+                unit
+            )
+        }))
+    } else {
+        None
+    }
 }
 
 pub fn get_rx_tx_data_points(
-    current_data: &DataCollection, network_scale_type: &AxisScaling, network_unit_type: &DataUnit,
-    network_use_binary_prefix: bool,
+    data: &DataCollection, scale_type: &AxisScaling, unit_type: &DataUnit, use_binary_prefix: bool,
 ) -> (Vec<Point>, Vec<Point>) {
     let mut rx: Vec<Point> = Vec::new();
     let mut tx: Vec<Point> = Vec::new();
 
-    let current_time = current_data.current_instant;
+    let current_time = data.current_instant;
 
-    for (time, data) in &current_data.timed_data_vec {
+    for (time, data) in &data.timed_data_vec {
         let time_from_start: f64 = (current_time.duration_since(*time).as_millis() as f64).floor();
 
-        let (rx_data, tx_data) = match network_scale_type {
+        let (rx_data, tx_data) = match scale_type {
             AxisScaling::Log => {
-                if network_use_binary_prefix {
-                    match network_unit_type {
+                if use_binary_prefix {
+                    match unit_type {
                         DataUnit::Byte => {
                             // As dividing by 8 is equal to subtracting 4 in base 2!
                             ((data.rx_data).log2() - 4.0, (data.tx_data).log2() - 4.0)
@@ -334,7 +325,7 @@ pub fn get_rx_tx_data_points(
                         DataUnit::Bit => ((data.rx_data).log2(), (data.tx_data).log2()),
                     }
                 } else {
-                    match network_unit_type {
+                    match unit_type {
                         DataUnit::Byte => {
                             ((data.rx_data / 8.0).log10(), (data.tx_data / 8.0).log10())
                         }
@@ -342,7 +333,7 @@ pub fn get_rx_tx_data_points(
                     }
                 }
             }
-            AxisScaling::Linear => match network_unit_type {
+            AxisScaling::Linear => match unit_type {
                 DataUnit::Byte => (data.rx_data / 8.0, data.tx_data / 8.0),
                 DataUnit::Bit => (data.rx_data, data.tx_data),
             },
@@ -359,38 +350,33 @@ pub fn get_rx_tx_data_points(
 }
 
 pub fn convert_network_data_points(
-    current_data: &DataCollection, need_four_points: bool, network_scale_type: &AxisScaling,
-    network_unit_type: &DataUnit, network_use_binary_prefix: bool,
+    data: &DataCollection, need_four_points: bool, scale_type: &AxisScaling, unit_type: &DataUnit,
+    use_binary_prefix: bool,
 ) -> ConvertedNetworkData {
-    let (rx, tx) = get_rx_tx_data_points(
-        current_data,
-        network_scale_type,
-        network_unit_type,
-        network_use_binary_prefix,
-    );
+    let (rx, tx) = get_rx_tx_data_points(data, scale_type, unit_type, use_binary_prefix);
 
-    let unit = match network_unit_type {
+    let unit = match unit_type {
         DataUnit::Byte => "B/s",
         DataUnit::Bit => "b/s",
     };
 
-    let (rx_data, tx_data, total_rx_data, total_tx_data) = match network_unit_type {
+    let (rx_data, tx_data, total_rx_data, total_tx_data) = match unit_type {
         DataUnit::Byte => (
-            current_data.network_harvest.rx / 8,
-            current_data.network_harvest.tx / 8,
-            current_data.network_harvest.total_rx / 8,
-            current_data.network_harvest.total_tx / 8,
+            data.network_harvest.rx / 8,
+            data.network_harvest.tx / 8,
+            data.network_harvest.total_rx / 8,
+            data.network_harvest.total_tx / 8,
         ),
         DataUnit::Bit => (
-            current_data.network_harvest.rx,
-            current_data.network_harvest.tx,
-            current_data.network_harvest.total_rx / 8, // We always make this bytes...
-            current_data.network_harvest.total_tx / 8,
+            data.network_harvest.rx,
+            data.network_harvest.tx,
+            data.network_harvest.total_rx / 8, // We always make this bytes...
+            data.network_harvest.total_tx / 8,
         ),
     };
 
     let (rx_converted_result, total_rx_converted_result): ((f64, String), (f64, &'static str)) =
-        if network_use_binary_prefix {
+        if use_binary_prefix {
             (
                 get_binary_prefix(rx_data, unit), /* If this isn't obvious why there's two functions, one you can configure the unit, the other is always bytes */
                 get_binary_bytes(total_rx_data),
@@ -403,7 +389,7 @@ pub fn convert_network_data_points(
         };
 
     let (tx_converted_result, total_tx_converted_result): ((f64, String), (f64, &'static str)) =
-        if network_use_binary_prefix {
+        if use_binary_prefix {
             (
                 get_binary_prefix(tx_data, unit),
                 get_binary_bytes(total_tx_data),
@@ -437,12 +423,12 @@ pub fn convert_network_data_points(
     } else {
         let rx_display = format!(
             "RX: {:<10}  All: {}",
-            if network_use_binary_prefix {
+            if use_binary_prefix {
                 format!("{:.1}{:3}", rx_converted_result.0, rx_converted_result.1)
             } else {
                 format!("{:.1}{:2}", rx_converted_result.0, rx_converted_result.1)
             },
-            if network_use_binary_prefix {
+            if use_binary_prefix {
                 format!(
                     "{:.1}{:3}",
                     total_rx_converted_result.0, total_rx_converted_result.1
@@ -456,12 +442,12 @@ pub fn convert_network_data_points(
         );
         let tx_display = format!(
             "TX: {:<10}  All: {}",
-            if network_use_binary_prefix {
+            if use_binary_prefix {
                 format!("{:.1}{:3}", tx_converted_result.0, tx_converted_result.1)
             } else {
                 format!("{:.1}{:2}", tx_converted_result.0, tx_converted_result.1)
             },
-            if network_use_binary_prefix {
+            if use_binary_prefix {
                 format!(
                     "{:.1}{:3}",
                     total_tx_converted_result.0, total_tx_converted_result.1
@@ -544,9 +530,20 @@ pub fn convert_battery_harvest(current_data: &DataCollection) -> Vec<ConvertedBa
             } else if let Some(secs) = battery_harvest.secs_until_full {
                 BatteryDuration::ToFull(secs)
             } else {
-                BatteryDuration::Unknown
+                match battery_harvest.state {
+                    starship_battery::State::Empty => BatteryDuration::Empty,
+                    starship_battery::State::Full => BatteryDuration::Full,
+                    _ => BatteryDuration::Unknown,
+                }
             },
             health: format!("{:.2}%", battery_harvest.health_percent),
+            state: {
+                let mut s = battery_harvest.state.to_string();
+                if !s.is_empty() {
+                    s[0..1].make_ascii_uppercase();
+                }
+                s
+            },
         })
         .collect()
 }
@@ -555,40 +552,20 @@ pub fn convert_battery_harvest(current_data: &DataCollection) -> Vec<ConvertedBa
 pub fn convert_arc_labels(
     current_data: &crate::app::data_farmer::DataCollection,
 ) -> Option<(String, String)> {
-    /// Returns the unit type and denominator for given total amount of memory in kibibytes.
-    fn return_unit_and_denominator_for_mem_kib(mem_total_kib: u64) -> (&'static str, f64) {
-        if mem_total_kib < 1024 {
-            // Stay with KiB
-            ("KiB", 1.0)
-        } else if mem_total_kib < MEBI_LIMIT {
-            // Use MiB
-            ("MiB", KIBI_LIMIT_F64)
-        } else if mem_total_kib < GIBI_LIMIT {
-            // Use GiB
-            ("GiB", MEBI_LIMIT_F64)
-        } else {
-            // Use TiB
-            ("TiB", GIBI_LIMIT_F64)
-        }
-    }
-
-    if current_data.arc_harvest.mem_total_in_kib > 0 {
+    if current_data.arc_harvest.total_bytes > 0 {
         Some((
             format!(
                 "{:3.0}%",
                 current_data.arc_harvest.use_percent.unwrap_or(0.0)
             ),
             {
-                let (unit, denominator) = return_unit_and_denominator_for_mem_kib(
-                    current_data.arc_harvest.mem_total_in_kib,
-                );
+                let (unit, denominator) =
+                    get_mem_binary_unit_and_denominator(current_data.arc_harvest.total_bytes);
 
                 format!(
-                    "   {:.1}{}/{:.1}{}",
-                    current_data.arc_harvest.mem_used_in_kib as f64 / denominator,
-                    unit,
-                    (current_data.arc_harvest.mem_total_in_kib as f64 / denominator),
-                    unit
+                    "   {:.1}{unit}/{:.1}{unit}",
+                    current_data.arc_harvest.used_bytes as f64 / denominator,
+                    (current_data.arc_harvest.total_bytes as f64 / denominator),
                 )
             },
         ))
@@ -631,23 +608,6 @@ pub struct ConvertedGpuData {
 pub fn convert_gpu_data(
     current_data: &crate::app::data_farmer::DataCollection,
 ) -> Option<Vec<ConvertedGpuData>> {
-    /// Returns the unit type and denominator for given total amount of memory in kibibytes.
-    fn return_unit_and_denominator_for_mem_kib(mem_total_kib: u64) -> (&'static str, f64) {
-        if mem_total_kib < 1024 {
-            // Stay with KiB
-            ("KiB", 1.0)
-        } else if mem_total_kib < MEBI_LIMIT {
-            // Use MiB
-            ("MiB", KIBI_LIMIT_F64)
-        } else if mem_total_kib < GIBI_LIMIT {
-            // Use GiB
-            ("GiB", MEBI_LIMIT_F64)
-        } else {
-            // Use TiB
-            ("TiB", GIBI_LIMIT_F64)
-        }
-    }
-
     let current_time = current_data.current_instant;
 
     // convert points
@@ -688,14 +648,12 @@ pub fn convert_gpu_data(
                 mem_percent: format!("{:3.0}%", gpu.1.use_percent.unwrap_or(0.0)),
                 mem_total: {
                     let (unit, denominator) =
-                        return_unit_and_denominator_for_mem_kib(gpu.1.mem_total_in_kib);
+                        get_mem_binary_unit_and_denominator(gpu.1.total_bytes);
 
                     format!(
-                        "   {:.1}{}/{:.1}{}",
-                        gpu.1.mem_used_in_kib as f64 / denominator,
-                        unit,
-                        (gpu.1.mem_total_in_kib as f64 / denominator),
-                        unit
+                        "   {:.1}{unit}/{:.1}{unit}",
+                        gpu.1.used_bytes as f64 / denominator,
+                        (gpu.1.total_bytes as f64 / denominator),
                     )
                 },
             }
