@@ -6,13 +6,15 @@
 //! bottom, refer to [here](https://clementtsang.github.io/bottom/stable/).
 
 #![warn(rust_2018_idioms)]
-#[allow(unused_imports)]
+#![deny(clippy::todo)]
+#![deny(clippy::unimplemented)]
+#![deny(clippy::missing_safety_doc)]
+#[allow(unused_imports)] // TODO: Deny unused imports.
+// Only used for builds not intended for release.
 #[cfg(feature = "log")]
 #[macro_use]
 extern crate log;
 
-// TODO: Deny unused imports.
-
 use std::{
     boxed::Box,
     fs,
@@ -32,13 +34,13 @@ use app::{
     data_harvester,
     frozen_state::FrozenState,
     layout_manager::{UsedWidgets, WidgetDirection},
-    App,
+    App, AppConfigFields, DataFilters,
 };
 use constants::*;
 use crossterm::{
     event::{
         poll, read, DisableBracketedPaste, DisableMouseCapture, Event, KeyCode, KeyEvent,
-        KeyModifiers, MouseEvent, MouseEventKind,
+        KeyEventKind, KeyModifiers, MouseEvent, MouseEventKind,
     },
     execute,
     style::Print,
@@ -82,7 +84,7 @@ pub enum BottomEvent {
 #[derive(Debug)]
 pub enum ThreadControlEvent {
     Reset,
-    UpdateConfig(Box<app::AppConfigFields>),
+    UpdateConfig(Box<AppConfigFields>),
     UpdateUsedWidgets(Box<UsedWidgets>),
     UpdateUpdateTime(u64),
 }
@@ -148,6 +150,8 @@ pub fn handle_key_event_or_break(
                 KeyCode::Char('c') | KeyCode::Char('C') => app.toggle_ignore_case(),
                 KeyCode::Char('w') | KeyCode::Char('W') => app.toggle_search_whole_word(),
                 KeyCode::Char('r') | KeyCode::Char('R') => app.toggle_search_regex(),
+                // KeyCode::Char('b') | KeyCode::Char('B') => todo!(),
+                // KeyCode::Char('f') | KeyCode::Char('F') => todo!(),
                 KeyCode::Char('h') => app.on_left_key(),
                 KeyCode::Char('l') => app.on_right_key(),
                 _ => {}
@@ -199,9 +203,9 @@ pub fn handle_key_event_or_break(
     false
 }
 
-pub fn read_config(config_location: Option<&str>) -> error::Result<Option<PathBuf>> {
+pub fn read_config(config_location: Option<&String>) -> error::Result<Option<PathBuf>> {
     let config_path = if let Some(conf_loc) = config_location {
-        Some(PathBuf::from(conf_loc))
+        Some(PathBuf::from(conf_loc.as_str()))
     } else if cfg!(target_os = "windows") {
         if let Some(home_path) = dirs::config_dir() {
             let mut path = home_path;
@@ -238,7 +242,7 @@ pub fn create_or_get_config(config_path: &Option<PathBuf>) -> error::Result<Conf
     if let Some(path) = config_path {
         if let Ok(config_string) = fs::read_to_string(path) {
             // We found a config file!
-            Ok(toml::from_str(config_string.as_str())?)
+            Ok(toml_edit::de::from_str(config_string.as_str())?)
         } else {
             // Config file DNE...
             if let Some(parent_path) = path.parent() {
@@ -382,6 +386,10 @@ pub fn update_data(app: &mut App) {
     // TODO: [OPT] Prefer reassignment over new vectors?
     if app.mem_state.force_update.is_some() {
         app.converted_data.mem_data = convert_mem_data_points(data_source);
+        #[cfg(not(target_os = "windows"))]
+        {
+            app.converted_data.cache_data = convert_cache_data_points(data_source);
+        }
         app.converted_data.swap_data = convert_swap_data_points(data_source);
         #[cfg(feature = "zfs")]
         {
@@ -440,7 +448,9 @@ pub fn create_input_thread(
                                     break;
                                 }
                             }
-                            Event::Key(key) => {
+                            Event::Key(key) if key.kind == KeyEventKind::Press => {
+                                // For now, we only care about keydown events. This may change in the future.
+
                                 if sender.send(BottomEvent::KeyInput(key)).is_err() {
                                     break;
                                 }
@@ -474,8 +484,7 @@ pub fn create_input_thread(
 pub fn create_collection_thread(
     sender: Sender<BottomEvent>, control_receiver: Receiver<ThreadControlEvent>,
     termination_ctrl_lock: Arc<Mutex<bool>>, termination_ctrl_cvar: Arc<Condvar>,
-    app_config_fields: &app::AppConfigFields, filters: app::DataFilters,
-    used_widget_set: UsedWidgets,
+    app_config_fields: &AppConfigFields, filters: DataFilters, used_widget_set: UsedWidgets,
 ) -> JoinHandle<()> {
     let temp_type = app_config_fields.temperature_type;
     let use_current_cpu_total = app_config_fields.use_current_cpu_total;
@@ -527,8 +536,7 @@ pub fn create_collection_thread(
                 }
             }
 
-            // TODO: [OPT] this feels like it might not be totally optimal. Hm.
-            futures::executor::block_on(data_state.update_data());
+            data_state.update_data();
 
             // Yet another check to bail if needed...
             if let Ok(is_terminated) = termination_ctrl_lock.try_lock() {
