@@ -1,38 +1,38 @@
 use std::{
     borrow::Cow,
-    collections::{HashMap, HashSet},
     convert::TryInto,
     str::FromStr,
     time::{Duration, Instant},
 };
 
 use clap::ArgMatches;
+use hashbrown::{HashMap, HashSet};
+use indexmap::IndexSet;
 use layout_options::*;
 use regex::Regex;
 use serde::{Deserialize, Serialize};
+use typed_builder::*;
 
 #[cfg(feature = "battery")]
 use starship_battery::Manager;
 
-use typed_builder::*;
-
 use crate::{
-    app::{layout_manager::*, *},
-    canvas::{canvas_colours::CanvasColours, ColourScheme},
+    app::{filter::Filter, layout_manager::*, *},
+    canvas::{canvas_styling::CanvasColours, ColourScheme},
     constants::*,
     units::data_units::DataUnit,
     utils::error::{self, BottomError},
-    widgets::{
-        BatteryWidgetState, CpuWidgetState, DiskTableWidget, MemWidgetState, NetWidgetState,
-        ProcWidgetMode, ProcWidgetState, TempWidgetState,
-    },
+    widgets::*,
 };
 
 pub mod layout_options;
 
+pub mod process_columns;
+use self::process_columns::ProcessConfig;
+
 use anyhow::{Context, Result};
 
-#[derive(Clone, Debug, Default, Deserialize, Serialize)]
+#[derive(Clone, Debug, Default, Deserialize)]
 pub struct Config {
     pub flags: Option<ConfigFlags>,
     pub colors: Option<ConfigColours>,
@@ -41,18 +41,7 @@ pub struct Config {
     pub mount_filter: Option<IgnoreList>,
     pub temp_filter: Option<IgnoreList>,
     pub net_filter: Option<IgnoreList>,
-}
-
-impl Config {
-    pub fn get_config_as_bytes(&self) -> anyhow::Result<Vec<u8>> {
-        let config_string: Vec<Cow<'_, str>> = vec![
-            // Top level
-            CONFIG_TOP_HEAD.into(),
-            toml::to_string_pretty(self)?.into(),
-        ];
-
-        Ok(config_string.concat().as_bytes().to_vec())
-    }
+    pub processes: Option<ProcessConfig>,
 }
 
 #[derive(Clone, Debug, Default, Deserialize, Serialize, TypedBuilder)]
@@ -81,7 +70,7 @@ pub struct ConfigFlags {
     pub battery: Option<bool>,
     pub disable_click: Option<bool>,
     pub no_write: Option<bool>,
-    // For built-in colour palettes.
+    /// For built-in colour palettes.
     pub color: Option<String>,
     pub mem_as_value: Option<bool>,
     pub tree: Option<bool>,
@@ -92,29 +81,12 @@ pub struct ConfigFlags {
     pub network_use_log: Option<bool>,
     pub network_use_binary_prefix: Option<bool>,
     pub enable_gpu_memory: Option<bool>,
+    pub enable_cache_memory: Option<bool>,
     #[serde(with = "humantime_serde")]
     #[serde(default)]
     pub retention: Option<Duration>,
 }
 
-#[derive(Clone, Default, Debug, Deserialize, Serialize)]
-pub struct WidgetIdEnabled {
-    id: u64,
-    enabled: bool,
-}
-
-impl WidgetIdEnabled {
-    pub fn create_from_hashmap(hashmap: &HashMap<u64, bool>) -> Vec<WidgetIdEnabled> {
-        hashmap
-            .iter()
-            .map(|(id, enabled)| WidgetIdEnabled {
-                id: *id,
-                enabled: *enabled,
-            })
-            .collect()
-    }
-}
-
 #[derive(Clone, Debug, Default, Deserialize, Serialize)]
 pub struct ConfigColours {
     pub table_header_color: Option<Cow<'static, str>>,
@@ -122,6 +94,8 @@ pub struct ConfigColours {
     pub avg_cpu_color: Option<Cow<'static, str>>,
     pub cpu_core_colors: Option<Vec<Cow<'static, str>>>,
     pub ram_color: Option<Cow<'static, str>>,
+    #[cfg(not(target_os = "windows"))]
+    pub cache_color: Option<Cow<'static, str>>,
     pub swap_color: Option<Cow<'static, str>>,
     pub arc_color: Option<Cow<'static, str>>,
     pub gpu_core_colors: Option<Vec<Cow<'static, str>>>,
@@ -143,11 +117,10 @@ pub struct ConfigColours {
 }
 
 impl ConfigColours {
+    /// Returns `true` if there is a [`ConfigColours`] that is empty or there isn't one at all.
     pub fn is_empty(&self) -> bool {
-        if let Ok(serialized_string) = toml::to_string(self) {
-            if !serialized_string.is_empty() {
-                return false;
-            }
+        if let Ok(serialized_string) = toml_edit::ser::to_string(self) {
+            return serialized_string.is_empty();
         }
 
         true
@@ -174,6 +147,28 @@ pub struct IgnoreList {
     pub whole_word: bool,
 }
 
+macro_rules! is_flag_enabled {
+    ($flag_name:ident, $matches:expr, $config:expr) => {
+        if $matches.get_flag(stringify!($flag_name)) {
+            true
+        } else if let Some(flags) = &$config.flags {
+            flags.$flag_name.unwrap_or(false)
+        } else {
+            false
+        }
+    };
+
+    ($cmd_flag:literal, $cfg_flag:ident, $matches:expr, $config:expr) => {
+        if $matches.get_flag($cmd_flag) {
+            true
+        } else if let Some(flags) = &$config.flags {
+            flags.$cfg_flag.unwrap_or(false)
+        } else {
+            false
+        }
+    };
+}
+
 pub fn build_app(
     matches: &ArgMatches, config: &mut Config, widget_layout: &BottomLayout,
     default_widget_id: u64, default_widget_type_option: &Option<BottomWidgetType>,
@@ -183,16 +178,18 @@ pub fn build_app(
 
     let retention_ms =
         get_retention_ms(matches, config).context("Update `retention` in your config file.")?;
-    let autohide_time = get_autohide_time(matches, config);
+    let autohide_time = is_flag_enabled!(autohide_time, matches, config);
     let default_time_value = get_default_time_value(matches, config, retention_ms)
         .context("Update 'default_time_value' in your config file.")?;
-    let use_basic_mode = get_use_basic_mode(matches, config);
+
+    let use_basic_mode = is_flag_enabled!(basic, matches, config);
+    let expanded_upon_startup = is_flag_enabled!(expanded_on_startup, matches, config);
 
     // For processes
-    let is_grouped = get_app_grouping(matches, config);
-    let is_case_sensitive = get_app_case_sensitive(matches, config);
-    let is_match_whole_word = get_app_match_whole_word(matches, config);
-    let is_use_regex = get_app_use_regex(matches, config);
+    let is_grouped = is_flag_enabled!("group", group_processes, matches, config);
+    let is_case_sensitive = is_flag_enabled!(case_sensitive, matches, config);
+    let is_match_whole_word = is_flag_enabled!(whole_word, matches, config);
+    let is_use_regex = is_flag_enabled!(regex, matches, config);
 
     let mut widget_map = HashMap::new();
     let mut cpu_state_map: HashMap<u64, CpuWidgetState> = HashMap::new();
@@ -214,14 +211,32 @@ pub fn build_app(
     let is_custom_layout = config.row.is_some();
     let mut used_widget_set = HashSet::new();
 
-    let show_memory_as_values = get_mem_as_value(matches, config);
-    let is_default_tree = get_is_default_tree(matches, config);
-    let is_default_command = get_is_default_process_command(matches, config);
-    let is_advanced_kill = !get_is_advanced_kill_disabled(matches, config);
+    let show_memory_as_values = is_flag_enabled!(mem_as_value, matches, config);
+    let is_default_tree = is_flag_enabled!(tree, matches, config);
+    let is_default_command = is_flag_enabled!(process_command, matches, config);
+    let is_advanced_kill = !(is_flag_enabled!(disable_advanced_kill, matches, config));
 
     let network_unit_type = get_network_unit_type(matches, config);
     let network_scale_type = get_network_scale_type(matches, config);
-    let network_use_binary_prefix = get_network_use_binary_prefix(matches, config);
+    let network_use_binary_prefix = is_flag_enabled!(network_use_binary_prefix, matches, config);
+
+    let proc_columns: Option<IndexSet<ProcWidgetColumn>> = {
+        let columns = config
+            .processes
+            .as_ref()
+            .and_then(|cfg| cfg.columns.clone());
+
+        match columns {
+            Some(columns) => {
+                if columns.is_empty() {
+                    None
+                } else {
+                    Some(IndexSet::from_iter(columns.into_iter()))
+                }
+            }
+            None => None,
+        }
+    };
 
     let app_config_fields = AppConfigFields {
         update_rate_in_milliseconds: get_update_rate_in_milliseconds(matches, config)
@@ -229,21 +244,22 @@ pub fn build_app(
         temperature_type: get_temperature(matches, config)
             .context("Update 'temperature_type' in your config file.")?,
         show_average_cpu: get_show_average_cpu(matches, config),
-        use_dot: get_use_dot(matches, config),
-        left_legend: get_use_left_legend(matches, config),
-        use_current_cpu_total: get_use_current_cpu_total(matches, config),
-        unnormalized_cpu: get_unnormalized_cpu(matches, config),
+        use_dot: is_flag_enabled!(dot_marker, matches, config),
+        left_legend: is_flag_enabled!(left_legend, matches, config),
+        use_current_cpu_total: is_flag_enabled!(current_usage, matches, config),
+        unnormalized_cpu: is_flag_enabled!(unnormalized_cpu, matches, config),
         use_basic_mode,
         default_time_value,
         time_interval: get_time_interval(matches, config, retention_ms)
             .context("Update 'time_delta' in your config file.")?,
-        hide_time: get_hide_time(matches, config),
+        hide_time: is_flag_enabled!(hide_time, matches, config),
         autohide_time,
-        use_old_network_legend: get_use_old_network_legend(matches, config),
-        table_gap: u16::from(!get_hide_table_gap(matches, config)),
-        disable_click: get_disable_click(matches, config),
+        use_old_network_legend: is_flag_enabled!(use_old_network_legend, matches, config),
+        table_gap: u16::from(!(is_flag_enabled!(hide_table_gap, matches, config))),
+        disable_click: is_flag_enabled!(disable_click, matches, config),
         enable_gpu_memory: get_enable_gpu_memory(matches, config),
-        show_table_scroll_position: get_show_table_scroll_position(matches, config),
+        enable_cache_memory: get_enable_cache_memory(matches, config),
+        show_table_scroll_position: is_flag_enabled!(show_table_scroll_position, matches, config),
         is_advanced_kill,
         network_scale_type,
         network_unit_type,
@@ -251,6 +267,14 @@ pub fn build_app(
         retention_ms,
     };
 
+    let table_config = ProcTableConfig {
+        is_case_sensitive,
+        is_match_whole_word,
+        is_use_regex,
+        show_memory_as_values,
+        is_command: is_default_command,
+    };
+
     for row in &widget_layout.rows {
         for col in &row.children {
             for col_row in &col.children {
@@ -329,12 +353,9 @@ pub fn build_app(
                                 ProcWidgetState::new(
                                     &app_config_fields,
                                     mode,
-                                    is_case_sensitive,
-                                    is_match_whole_word,
-                                    is_use_regex,
-                                    show_memory_as_values,
-                                    is_default_command,
+                                    table_config,
                                     colours,
+                                    &proc_columns,
                                 ),
                             );
                         }
@@ -390,6 +411,7 @@ pub fn build_app(
     let used_widgets = UsedWidgets {
         use_cpu: used_widget_set.get(&Cpu).is_some() || used_widget_set.get(&BasicCpu).is_some(),
         use_mem,
+        use_cache: use_mem && get_enable_cache_memory(matches, config),
         use_gpu: use_mem && get_enable_gpu_memory(matches, config),
         use_net: used_widget_set.get(&Net).is_some() || used_widget_set.get(&BasicNet).is_some(),
         use_proc: used_widget_set.get(&Proc).is_some(),
@@ -407,8 +429,6 @@ pub fn build_app(
     let net_filter =
         get_ignore_list(&config.net_filter).context("Update 'net_filter' in your config file")?;
 
-    let expanded_upon_startup = get_expanded_on_startup(matches, config);
-
     Ok(App::builder()
         .app_config_fields(app_config_fields)
         .cpu_state(CpuState::init(cpu_state_map))
@@ -435,12 +455,13 @@ pub fn build_app(
 pub fn get_widget_layout(
     matches: &ArgMatches, config: &Config,
 ) -> error::Result<(BottomLayout, u64, Option<BottomWidgetType>)> {
-    let left_legend = get_use_left_legend(matches, config);
+    let left_legend = is_flag_enabled!(left_legend, matches, config);
+
     let (default_widget_type, mut default_widget_count) =
         get_default_widget_and_count(matches, config)?;
     let mut default_widget_id = 1;
 
-    let bottom_layout = if get_use_basic_mode(matches, config) {
+    let bottom_layout = if is_flag_enabled!(basic, matches, config) {
         default_widget_id = DEFAULT_WIDGET_ID;
 
         BottomLayout::init_basic_default(get_use_battery(matches, config))
@@ -450,7 +471,7 @@ pub fn get_widget_layout(
             Some(r) => r,
             None => {
                 // This cannot (like it really shouldn't) fail!
-                ref_row = toml::from_str::<Config>(if get_use_battery(matches, config) {
+                ref_row = toml_edit::de::from_str::<Config>(if get_use_battery(matches, config) {
                     DEFAULT_BATTERY_LAYOUT
                 } else {
                     DEFAULT_LAYOUT
@@ -498,7 +519,7 @@ pub fn get_widget_layout(
 }
 
 fn get_update_rate_in_milliseconds(matches: &ArgMatches, config: &Config) -> error::Result<u64> {
-    let update_rate_in_milliseconds = if let Some(update_rate) = matches.value_of("rate") {
+    let update_rate_in_milliseconds = if let Some(update_rate) = matches.get_one::<String>("rate") {
         update_rate.parse::<u64>().map_err(|_| {
             BottomError::ConfigError(
                 "could not parse as a valid 64-bit unsigned integer".to_string(),
@@ -526,11 +547,11 @@ fn get_update_rate_in_milliseconds(matches: &ArgMatches, config: &Config) -> err
 fn get_temperature(
     matches: &ArgMatches, config: &Config,
 ) -> error::Result<data_harvester::temperature::TemperatureType> {
-    if matches.is_present("fahrenheit") {
+    if matches.get_flag("fahrenheit") {
         return Ok(data_harvester::temperature::TemperatureType::Fahrenheit);
-    } else if matches.is_present("kelvin") {
+    } else if matches.get_flag("kelvin") {
         return Ok(data_harvester::temperature::TemperatureType::Kelvin);
-    } else if matches.is_present("celsius") {
+    } else if matches.get_flag("celsius") {
         return Ok(data_harvester::temperature::TemperatureType::Celsius);
     } else if let Some(flags) = &config.flags {
         if let Some(temp_type) = &flags.temperature_type {
@@ -551,7 +572,7 @@ fn get_temperature(
 
 /// Yes, this function gets whether to show average CPU (true) or not (false)
 fn get_show_average_cpu(matches: &ArgMatches, config: &Config) -> bool {
-    if matches.is_present("hide_avg_cpu") {
+    if matches.get_flag("hide_avg_cpu") {
         return false;
     } else if let Some(flags) = &config.flags {
         if let Some(avg_cpu) = flags.hide_avg_cpu {
@@ -562,84 +583,26 @@ fn get_show_average_cpu(matches: &ArgMatches, config: &Config) -> bool {
     true
 }
 
-fn get_use_dot(matches: &ArgMatches, config: &Config) -> bool {
-    if matches.is_present("dot_marker") {
-        return true;
-    } else if let Some(flags) = &config.flags {
-        if let Some(dot_marker) = flags.dot_marker {
-            return dot_marker;
-        }
-    }
-    false
-}
-
-fn get_use_left_legend(matches: &ArgMatches, config: &Config) -> bool {
-    if matches.is_present("left_legend") {
-        return true;
-    } else if let Some(flags) = &config.flags {
-        if let Some(left_legend) = flags.left_legend {
-            return left_legend;
-        }
-    }
-
-    false
-}
-
-fn get_use_current_cpu_total(matches: &ArgMatches, config: &Config) -> bool {
-    if matches.is_present("current_usage") {
-        return true;
-    } else if let Some(flags) = &config.flags {
-        if let Some(current_usage) = flags.current_usage {
-            return current_usage;
-        }
-    }
-
-    false
-}
-
-fn get_unnormalized_cpu(matches: &ArgMatches, config: &Config) -> bool {
-    if matches.is_present("unnormalized_cpu") {
-        return true;
-    } else if let Some(flags) = &config.flags {
-        if let Some(unnormalized_cpu) = flags.unnormalized_cpu {
-            return unnormalized_cpu;
-        }
-    }
-
-    false
-}
-
-fn get_use_basic_mode(matches: &ArgMatches, config: &Config) -> bool {
-    if matches.is_present("basic") {
-        return true;
-    } else if let Some(flags) = &config.flags {
-        if let Some(basic) = flags.basic {
-            return basic;
-        }
-    }
-
-    false
-}
-
 /// FIXME: Let this accept human times.
 fn get_default_time_value(
     matches: &ArgMatches, config: &Config, retention_ms: u64,
 ) -> error::Result<u64> {
-    let default_time = if let Some(default_time_value) = matches.value_of("default_time_value") {
-        default_time_value.parse::<u64>().map_err(|_| {
-            BottomError::ConfigError(
-                "could not parse as a valid 64-bit unsigned integer".to_string(),
-            )
-        })?
-    } else if let Some(flags) = &config.flags {
-        if let Some(default_time_value) = flags.default_time_value {
-            default_time_value
+    let default_time =
+        if let Some(default_time_value) = matches.get_one::<String>("default_time_value") {
+            default_time_value.parse::<u64>().map_err(|_| {
+                BottomError::ConfigError(
+                    "could not parse as a valid 64-bit unsigned integer".to_string(),
+                )
+            })?
+        } else if let Some(flags) = &config.flags {
+            if let Some(default_time_value) = flags.default_time_value {
+                default_time_value
+            } else {
+                DEFAULT_TIME_MILLISECONDS
+            }
         } else {
             DEFAULT_TIME_MILLISECONDS
-        }
-    } else {
-        DEFAULT_TIME_MILLISECONDS
-    };
+        };
 
     if default_time < 30000 {
         return Err(BottomError::ConfigError(
@@ -658,7 +621,7 @@ fn get_default_time_value(
 fn get_time_interval(
     matches: &ArgMatches, config: &Config, retention_ms: u64,
 ) -> error::Result<u64> {
-    let time_interval = if let Some(time_interval) = matches.value_of("time_delta") {
+    let time_interval = if let Some(time_interval) = matches.get_one::<String>("time_delta") {
         time_interval.parse::<u64>().map_err(|_| {
             BottomError::ConfigError(
                 "could not parse as a valid 64-bit unsigned integer".to_string(),
@@ -688,86 +651,10 @@ fn get_time_interval(
     Ok(time_interval)
 }
 
-pub fn get_app_grouping(matches: &ArgMatches, config: &Config) -> bool {
-    if matches.is_present("group") {
-        return true;
-    } else if let Some(flags) = &config.flags {
-        if let Some(grouping) = flags.group_processes {
-            return grouping;
-        }
-    }
-    false
-}
-
-pub fn get_app_case_sensitive(matches: &ArgMatches, config: &Config) -> bool {
-    if matches.is_present("case_sensitive") {
-        return true;
-    } else if let Some(flags) = &config.flags {
-        if let Some(case_sensitive) = flags.case_sensitive {
-            return case_sensitive;
-        }
-    }
-    false
-}
-
-pub fn get_app_match_whole_word(matches: &ArgMatches, config: &Config) -> bool {
-    if matches.is_present("whole_word") {
-        return true;
-    } else if let Some(flags) = &config.flags {
-        if let Some(whole_word) = flags.whole_word {
-            return whole_word;
-        }
-    }
-    false
-}
-
-pub fn get_app_use_regex(matches: &ArgMatches, config: &Config) -> bool {
-    if matches.is_present("regex") {
-        return true;
-    } else if let Some(flags) = &config.flags {
-        if let Some(regex) = flags.regex {
-            return regex;
-        }
-    }
-    false
-}
-
-fn get_hide_time(matches: &ArgMatches, config: &Config) -> bool {
-    if matches.is_present("hide_time") {
-        return true;
-    } else if let Some(flags) = &config.flags {
-        if let Some(hide_time) = flags.hide_time {
-            return hide_time;
-        }
-    }
-    false
-}
-
-fn get_autohide_time(matches: &ArgMatches, config: &Config) -> bool {
-    if matches.is_present("autohide_time") {
-        return true;
-    } else if let Some(flags) = &config.flags {
-        if let Some(autohide_time) = flags.autohide_time {
-            return autohide_time;
-        }
-    }
-
-    false
-}
-
-fn get_expanded_on_startup(matches: &ArgMatches, config: &Config) -> bool {
-    matches.is_present("expanded_on_startup")
-        || config
-            .flags
-            .as_ref()
-            .and_then(|x| x.expanded_on_startup)
-            .unwrap_or(false)
-}
-
 fn get_default_widget_and_count(
     matches: &ArgMatches, config: &Config,
 ) -> error::Result<(Option<BottomWidgetType>, u64)> {
-    let widget_type = if let Some(widget_type) = matches.value_of("default_widget_type") {
+    let widget_type = if let Some(widget_type) = matches.get_one::<String>("default_widget_type") {
         let parsed_widget = widget_type.parse::<BottomWidgetType>()?;
         if let BottomWidgetType::Empty = parsed_widget {
             None
@@ -789,7 +676,8 @@ fn get_default_widget_and_count(
         None
     };
 
-    let widget_count = if let Some(widget_count) = matches.value_of("default_widget_count") {
+    let widget_count = if let Some(widget_count) = matches.get_one::<String>("default_widget_count")
+    {
         Some(widget_count.parse::<u128>()?)
     } else if let Some(flags) = &config.flags {
         flags
@@ -814,51 +702,19 @@ fn get_default_widget_and_count(
     }
 }
 
-fn get_disable_click(matches: &ArgMatches, config: &Config) -> bool {
-    if matches.is_present("disable_click") {
-        return true;
-    } else if let Some(flags) = &config.flags {
-        if let Some(disable_click) = flags.disable_click {
-            return disable_click;
-        }
-    }
-    false
-}
-
-fn get_use_old_network_legend(matches: &ArgMatches, config: &Config) -> bool {
-    if matches.is_present("use_old_network_legend") {
-        return true;
-    } else if let Some(flags) = &config.flags {
-        if let Some(use_old_network_legend) = flags.use_old_network_legend {
-            return use_old_network_legend;
-        }
-    }
-    false
-}
-
-fn get_hide_table_gap(matches: &ArgMatches, config: &Config) -> bool {
-    if matches.is_present("hide_table_gap") {
-        return true;
-    } else if let Some(flags) = &config.flags {
-        if let Some(hide_table_gap) = flags.hide_table_gap {
-            return hide_table_gap;
-        }
-    }
-    false
-}
-
+#[allow(unused_variables)]
 fn get_use_battery(matches: &ArgMatches, config: &Config) -> bool {
     #[cfg(feature = "battery")]
-    if let Ok(battery_manager) = Manager::new() {
-        if let Ok(batteries) = battery_manager.batteries() {
-            if batteries.count() == 0 {
-                return false;
+    {
+        if let Ok(battery_manager) = Manager::new() {
+            if let Ok(batteries) = battery_manager.batteries() {
+                if batteries.count() == 0 {
+                    return false;
+                }
             }
         }
-    }
 
-    if cfg!(feature = "battery") {
-        if matches.is_present("battery") {
+        if matches.get_flag("battery") {
             return true;
         } else if let Some(flags) = &config.flags {
             if let Some(battery) = flags.battery {
@@ -866,12 +722,15 @@ fn get_use_battery(matches: &ArgMatches, config: &Config) -> bool {
             }
         }
     }
+
     false
 }
 
+#[allow(unused_variables)]
 fn get_enable_gpu_memory(matches: &ArgMatches, config: &Config) -> bool {
-    if cfg!(feature = "gpu") {
-        if matches.is_present("enable_gpu_memory") {
+    #[cfg(feature = "gpu")]
+    {
+        if matches.get_flag("enable_gpu_memory") {
             return true;
         } else if let Some(flags) = &config.flags {
             if let Some(enable_gpu_memory) = flags.enable_gpu_memory {
@@ -879,18 +738,23 @@ fn get_enable_gpu_memory(matches: &ArgMatches, config: &Config) -> bool {
             }
         }
     }
+
     false
 }
 
-#[allow(dead_code)]
-fn get_no_write(matches: &ArgMatches, config: &Config) -> bool {
-    if matches.is_present("no_write") {
-        return true;
-    } else if let Some(flags) = &config.flags {
-        if let Some(no_write) = flags.no_write {
-            return no_write;
+#[allow(unused_variables)]
+fn get_enable_cache_memory(matches: &ArgMatches, config: &Config) -> bool {
+    #[cfg(not(target_os = "windows"))]
+    {
+        if matches.get_flag("enable_cache_memory") {
+            return true;
+        } else if let Some(flags) = &config.flags {
+            if let Some(enable_cache_memory) = flags.enable_cache_memory {
+                return enable_cache_memory;
+            }
         }
     }
+
     false
 }
 
@@ -932,7 +796,7 @@ fn get_ignore_list(ignore_list: &Option<IgnoreList>) -> error::Result<Option<Fil
 }
 
 pub fn get_color_scheme(matches: &ArgMatches, config: &Config) -> error::Result<ColourScheme> {
-    if let Some(color) = matches.value_of("color") {
+    if let Some(color) = matches.get_one::<String>("color") {
         // Highest priority is always command line flags...
         return ColourScheme::from_str(color);
     } else if let Some(colors) = &config.colors {
@@ -956,63 +820,8 @@ pub fn get_color_scheme(matches: &ArgMatches, config: &Config) -> error::Result<
     Ok(ColourScheme::Default)
 }
 
-fn get_mem_as_value(matches: &ArgMatches, config: &Config) -> bool {
-    if matches.is_present("mem_as_value") {
-        return true;
-    } else if let Some(flags) = &config.flags {
-        if let Some(mem_as_value) = flags.mem_as_value {
-            return mem_as_value;
-        }
-    }
-    false
-}
-
-fn get_is_default_tree(matches: &ArgMatches, config: &Config) -> bool {
-    if matches.is_present("tree") {
-        return true;
-    } else if let Some(flags) = &config.flags {
-        if let Some(tree) = flags.tree {
-            return tree;
-        }
-    }
-    false
-}
-
-fn get_show_table_scroll_position(matches: &ArgMatches, config: &Config) -> bool {
-    if matches.is_present("show_table_scroll_position") {
-        return true;
-    } else if let Some(flags) = &config.flags {
-        if let Some(show_table_scroll_position) = flags.show_table_scroll_position {
-            return show_table_scroll_position;
-        }
-    }
-    false
-}
-
-fn get_is_default_process_command(matches: &ArgMatches, config: &Config) -> bool {
-    if matches.is_present("process_command") {
-        return true;
-    } else if let Some(flags) = &config.flags {
-        if let Some(process_command) = flags.process_command {
-            return process_command;
-        }
-    }
-    false
-}
-
-fn get_is_advanced_kill_disabled(matches: &ArgMatches, config: &Config) -> bool {
-    if matches.is_present("disable_advanced_kill") {
-        return true;
-    } else if let Some(flags) = &config.flags {
-        if let Some(disable_advanced_kill) = flags.disable_advanced_kill {
-            return disable_advanced_kill;
-        }
-    }
-    false
-}
-
 fn get_network_unit_type(matches: &ArgMatches, config: &Config) -> DataUnit {
-    if matches.is_present("network_use_bytes") {
+    if matches.get_flag("network_use_bytes") {
         return DataUnit::Byte;
     } else if let Some(flags) = &config.flags {
         if let Some(network_use_bytes) = flags.network_use_bytes {
@@ -1026,7 +835,7 @@ fn get_network_unit_type(matches: &ArgMatches, config: &Config) -> DataUnit {
 }
 
 fn get_network_scale_type(matches: &ArgMatches, config: &Config) -> AxisScaling {
-    if matches.is_present("network_use_log") {
+    if matches.get_flag("network_use_log") {
         return AxisScaling::Log;
     } else if let Some(flags) = &config.flags {
         if let Some(network_use_log) = flags.network_use_log {
@@ -1039,21 +848,10 @@ fn get_network_scale_type(matches: &ArgMatches, config: &Config) -> AxisScaling
     AxisScaling::Linear
 }
 
-fn get_network_use_binary_prefix(matches: &ArgMatches, config: &Config) -> bool {
-    if matches.is_present("network_use_binary_prefix") {
-        return true;
-    } else if let Some(flags) = &config.flags {
-        if let Some(network_use_binary_prefix) = flags.network_use_binary_prefix {
-            return network_use_binary_prefix;
-        }
-    }
-    false
-}
-
 fn get_retention_ms(matches: &ArgMatches, config: &Config) -> error::Result<u64> {
     const DEFAULT_RETENTION_MS: u64 = 600 * 1000; // Keep 10 minutes of data.
 
-    if let Some(retention) = matches.value_of("retention") {
+    if let Some(retention) = matches.get_one::<String>("retention") {
         humantime::parse_duration(retention)
             .map(|dur| dur.as_millis() as u64)
             .map_err(|err| BottomError::ConfigError(format!("invalid retention duration: {err:?}")))
@@ -1067,3 +865,70 @@ fn get_retention_ms(matches: &ArgMatches, config: &Config) -> error::Result<u64>
         Ok(DEFAULT_RETENTION_MS)
     }
 }
+
+#[cfg(test)]
+mod test {
+
+    use clap::ArgMatches;
+
+    use super::{get_color_scheme, get_widget_layout, Config};
+    use crate::{app::App, canvas::canvas_styling::CanvasColours};
+
+    fn create_app(mut config: Config, matches: ArgMatches) -> App {
+        let (layout, id, ty) = get_widget_layout(&matches, &config).unwrap();
+        let colours =
+            CanvasColours::new(get_color_scheme(&matches, &config).unwrap(), &config).unwrap();
+
+        super::build_app(&matches, &mut config, &layout, id, &ty, &colours).unwrap()
+    }
+
+    // TODO: There's probably a better way to create clap options AND unify together to avoid the possibility of
+    // typos/mixing up. Use proc macros to unify on one struct?
+    #[test]
+    fn verify_cli_options_build() {
+        let app = crate::clap::build_app();
+
+        let default_app = {
+            let app = app.clone();
+            let config = Config::default();
+            let matches = app.get_matches_from([""]);
+
+            create_app(config, matches)
+        };
+
+        // Skip battery since it's tricky to test depending on the platform/features we're testing with.
+        let skip = ["help", "version", "celsius", "battery"];
+
+        for arg in app.get_arguments().collect::<Vec<_>>() {
+            let arg_name = arg
+                .get_long_and_visible_aliases()
+                .unwrap()
+                .first()
+                .unwrap()
+                .to_owned();
+
+            if !arg.get_action().takes_values() && !skip.contains(&arg_name) {
+                let arg = format!("--{arg_name}");
+
+                let arguments = vec!["btm", &arg];
+                let app = app.clone();
+                let config = Config::default();
+                let matches = app.get_matches_from(arguments);
+
+                let testing_app = create_app(config, matches);
+
+                if (default_app.app_config_fields == testing_app.app_config_fields)
+                    && default_app.is_expanded == testing_app.is_expanded
+                    && default_app
+                        .proc_state
+                        .widget_states
+                        .iter()
+                        .zip(testing_app.proc_state.widget_states.iter())
+                        .all(|(a, b)| (a.1.test_equality(b.1)))
+                {
+                    panic!("failed on {arg_name}");
+                }
+            }
+        }
+    }
+}
