@@ -14,9 +14,6 @@ pub enum BottomError {
     /// An error when there is an IO exception.
     #[error("IO exception, {0}")]
     InvalidIo(String),
-    /// An error when the heim library encounters a problem.
-    #[error("Error caused by Heim, {0}")]
-    InvalidHeim(String),
     /// An error when the Crossterm library encounters a problem.
     #[error("Error caused by Crossterm, {0}")]
     CrosstermError(String),
@@ -38,6 +35,8 @@ pub enum BottomError {
     /// An error that just signifies something minor went wrong; no message.
     #[error("Minor error.")]
     MinorError,
+    #[error("Error casting integers {0}")]
+    TryFromIntError(#[from] std::num::TryFromIntError),
     /// An error to represent errors with procfs
     #[cfg(target_os = "linux")]
     #[error("Procfs error, {0}")]
@@ -50,13 +49,6 @@ impl From<std::io::Error> for BottomError {
     }
 }
 
-#[cfg(not(target_os = "freebsd"))]
-impl From<heim::Error> for BottomError {
-    fn from(err: heim::Error) -> Self {
-        BottomError::InvalidHeim(err.to_string())
-    }
-}
-
 impl From<std::num::ParseIntError> for BottomError {
     fn from(err: std::num::ParseIntError) -> Self {
         BottomError::ConfigError(err.to_string())
@@ -69,8 +61,8 @@ impl From<std::string::String> for BottomError {
     }
 }
 
-impl From<toml::de::Error> for BottomError {
-    fn from(err: toml::de::Error) -> Self {
+impl From<toml_edit::de::Error> for BottomError {
+    fn from(err: toml_edit::de::Error) -> Self {
         BottomError::ConfigError(err.to_string())
     }
 }
