@@ -9,18 +9,18 @@ pub const MEGA_LIMIT: u64 = 1_000_000;
 pub const GIGA_LIMIT: u64 = 1_000_000_000;
 pub const TERA_LIMIT: u64 = 1_000_000_000_000;
 pub const KIBI_LIMIT: u64 = 1024;
-pub const MEBI_LIMIT: u64 = 1_048_576;
-pub const GIBI_LIMIT: u64 = 1_073_741_824;
-pub const TEBI_LIMIT: u64 = 1_099_511_627_776;
+pub const MEBI_LIMIT: u64 = 1024 * 1024;
+pub const GIBI_LIMIT: u64 = 1024 * 1024 * 1024;
+pub const TEBI_LIMIT: u64 = 1024 * 1024 * 1024 * 1024;
 
 pub const KILO_LIMIT_F64: f64 = 1000.0;
 pub const MEGA_LIMIT_F64: f64 = 1_000_000.0;
 pub const GIGA_LIMIT_F64: f64 = 1_000_000_000.0;
 pub const TERA_LIMIT_F64: f64 = 1_000_000_000_000.0;
 pub const KIBI_LIMIT_F64: f64 = 1024.0;
-pub const MEBI_LIMIT_F64: f64 = 1_048_576.0;
-pub const GIBI_LIMIT_F64: f64 = 1_073_741_824.0;
-pub const TEBI_LIMIT_F64: f64 = 1_099_511_627_776.0;
+pub const MEBI_LIMIT_F64: f64 = 1024.0 * 1024.0;
+pub const GIBI_LIMIT_F64: f64 = 1024.0 * 1024.0 * 1024.0;
+pub const TEBI_LIMIT_F64: f64 = 1024.0 * 1024.0 * 1024.0 * 1024.0;
 
 pub const LOG_KILO_LIMIT: f64 = 3.0;
 pub const LOG_MEGA_LIMIT: f64 = 6.0;
